
#include <asf.h>
#include <string.h>
#include <stringz.h>
#include <stdlib.h>
#include <inc/ringBufS.h>

#define USART1_HOSTLINK					  (&AVR32_USART1)		// HOST link
#define USART1_SERIAL_BAUDRATE            38400					// !!!!!!!!!!!!!!!!!!!!!!!!!! CHANGE BACK TO 57600!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#define USART1_SERIAL_CHAR_LENGTH         8
#define USART1_SERIAL_PARITY              USART_NO_PARITY
#define USART1_SERIAL_STOP_BIT            USART_1_STOPBIT
#define USART1_HOST_IRQ					  AVR32_USART1_IRQ

#define USART2_MAVLINK					  (&AVR32_USART2)		// MAV link
#define USART2_SERIAL_BAUDRATE            38400
#define USART2_SERIAL_CHAR_LENGTH         8
#define USART2_SERIAL_PARITY              USART_NO_PARITY
#define USART2_SERIAL_STOP_BIT            USART_1_STOPBIT
#define USART2_MAV_IRQ					  AVR32_USART2_IRQ


#define LED1ON 		gpio_set_pin_low(AVR32_PIN_PA08)
#define LED2ON		gpio_set_pin_low(AVR32_PIN_PA07)
#define LED1OFF 	gpio_set_pin_high(AVR32_PIN_PA08)
#define LED2OFF 	gpio_set_pin_high(AVR32_PIN_PA07)
#define LED1TOGGLE	gpio_toggle_pin(AVR32_PIN_PA08)
#define LED2TOGGLE	gpio_toggle_pin(AVR32_PIN_PA07)

usart_options_t usart1_options={		// HOST link
	.baudrate = USART1_SERIAL_BAUDRATE,
	.charlength = USART1_SERIAL_CHAR_LENGTH,
	.paritytype = USART1_SERIAL_PARITY,
	.stopbits = USART1_SERIAL_STOP_BIT,
	.channelmode = USART_NORMAL_CHMODE,
};
usart_options_t usart2_options={		// MAV link
	.baudrate = USART2_SERIAL_BAUDRATE,
	.charlength = USART2_SERIAL_CHAR_LENGTH,
	.paritytype = USART2_SERIAL_PARITY,
	.stopbits = USART2_SERIAL_STOP_BIT,
	.channelmode = USART_NORMAL_CHMODE,
};

ringBufS	rx_ringBuff;

__attribute__((__interrupt__)) static void usart_int_handler(void);

uint8_t received_byte;
uint8_t txdata;
uint8_t rxdata;

static const gpio_map_t ADC_GPIO_MAP= {
	{AVR32_ADC_AD_3_PIN,AVR32_ADC_AD_3_FUNCTION}
};
static const gpio_map_t USART0_GPIO_MAP = {
	//{AVR32_USART0_CLK_0_1_PIN,AVR32_USART0_CLK_0_1_FUNCTION},
	{AVR32_USART0_TXD_0_0_PIN,AVR32_USART0_TXD_0_0_FUNCTION},
	{AVR32_USART0_RXD_0_0_PIN,AVR32_USART0_RXD_0_0_FUNCTION},
};
static const gpio_map_t USART1_GPIO_MAP = {		// HOST link
	//{AVR32_USART1_CLK_0_PIN,AVR32_USART1_CLK_0_FUNCTION},
	{AVR32_USART1_TXD_0_0_PIN,AVR32_USART1_TXD_0_0_FUNCTION},
	{AVR32_USART1_RXD_0_0_PIN,AVR32_USART1_RXD_0_0_FUNCTION},
};
static const gpio_map_t USART2_GPIO_MAP = {		// MAV link
	//{AVR32_USART2_CLK_0_PIN,AVR32_USART2_CLK_0_FUNCTION},
	{AVR32_USART2_TXD_0_1_PIN,AVR32_USART2_TXD_0_1_FUNCTION},		// changed from AVR32_USART2_TXD_0_*0*_PIN
	{AVR32_USART2_RXD_0_1_PIN,AVR32_USART2_RXD_0_1_FUNCTION},		// changed from AVR32_USART2_RXD_0_*0*_PIN
};
 

int main (void)
{
	sysclk_init();				//Initializes Clock
	board_init();				//Initializes Board
	
	usart_serial_init(USART1_HOSTLINK, &usart1_options);		//Initializes USART1 (HOST)
	usart_serial_init(USART2_MAVLINK, &usart2_options);			//Initializes USART2 (MAV)
	gpio_enable_module(USART1_GPIO_MAP,sizeof(USART1_GPIO_MAP)/sizeof(USART1_GPIO_MAP[0]));		//enables USART1 Pins
	gpio_enable_module(USART2_GPIO_MAP,sizeof(USART2_GPIO_MAP)/sizeof(USART2_GPIO_MAP[0]));		//enables USART2 Pins
	
	AVR32_ADC.mr |= 0x4 << AVR32_ADC_MR_PRESCAL_OFFSET;									//Sets Prescaler for ADC clk
	gpio_enable_module(ADC_GPIO_MAP,sizeof(ADC_GPIO_MAP)/sizeof(ADC_GPIO_MAP[0]));		//Enables ADC Pin
	adc_configure(&AVR32_ADC);															//Initializes ADC
	adc_enable(&AVR32_ADC,AVR32_ADC_CH3);												//Enables channel 3 of the ADC
	
	gpio_configure_pin(AVR32_PIN_PA07,GPIO_DIR_OUTPUT);
	gpio_configure_pin(AVR32_PIN_PA08,GPIO_DIR_OUTPUT);
	LED1OFF; LED2OFF;
		
	Disable_global_interrupt();						
	 INTC_init_interrupts();
	 INTC_register_interrupt(&usart_int_handler, USART2_MAV_IRQ, AVR32_INTC_INT0);	// Register handler - IRQ is interrupt, INT0 is priority
     USART2_MAVLINK->ier =AVR32_USART_IER_RXRDY_MASK;	// Enable USART Rx interrupt.
    Enable_global_interrupt();							// Enable all interrupts.
	
	ringBufS_init(&rx_ringBuff);
	
	while(1) {
		
		
		if (ringBufS_full(&rx_ringBuff)) {
			LED1ON;
			usart_write_char(USART1_HOSTLINK,rx_ringBuff.count);
		}
		else {
			LED1OFF;	
		}
		int null=ringBufS_get(&rx_ringBuff);	
		delay_us(300);
	}
} // end MAIN()

 __attribute__((__interrupt__)) static void usart_int_handler(void)
 {
	 int c;
	 int status = usart_read_char(USART2_MAVLINK, &c);
	 if ((status == USART_RX_ERROR) || (status==USART_RX_EMPTY)) {
		 usart_reset_status(USART2_MAVLINK);
		 c=0xFF;
	 }
	 ringBufS_put(&rx_ringBuff, (unsigned char)c);
	// usart_write_char(USART1_HOSTLINK,rx_ringBuff.count);
 }
 
