
#include <asf.h>
#include "inc/mavlink_bridge_header.h"
#include "inc/mavlink1/common/mavlink.h"
#include <string.h>
#include <stringz.h>
#include <stdlib.h>
#include <inc/ringBufS.h>
						 

#define SPI_EXAMPLE						  &AVR32_SPI
#define SPI_SLAVECHIP_NUMBER			  0
#define IDLE							  0
#define SAMPLING						  1
#define LINKED							  2
#define TEST							  3

#define USART1_HOSTLINK					  &AVR32_USART1		// HOST link
#define USART1_SERIAL_BAUDRATE            57600
#define USART1_SERIAL_CHAR_LENGTH         8
#define USART1_SERIAL_PARITY              USART_NO_PARITY
#define USART1_SERIAL_STOP_BIT            USART_1_STOPBIT
#define USART1_HOST_IRQ					  AVR32_USART1_IRQ

#define USART2_MAVLINK					  &AVR32_USART2		// MAV link
#define USART2_SERIAL_BAUDRATE            38400
#define USART2_SERIAL_CHAR_LENGTH         8
#define USART2_SERIAL_PARITY              USART_NO_PARITY
#define USART2_SERIAL_STOP_BIT            USART_1_STOPBIT
#define USART2_MAV_IRQ					  AVR32_USART2_IRQ

#define MAVLINK_COMM_NUM_BUFFERS 1
#define PIX_SYSID	1
#define PIX_CMPID	1
#define STREAM_START	1
#define STREAM_STOP		0

#define FLTMODE_STAB	0
#define FLTMODE_MISSION	3
#define FLTMODE_LOITER	5
#define FLTMODE_RTL		6
#define FLTMODE_LAND	9
#define FLTMODE_HOLD	10

#define LED1ON 		gpio_set_pin_low(AVR32_PIN_PA08)
#define LED2ON		gpio_set_pin_low(AVR32_PIN_PA07)
#define LED1OFF 	gpio_set_pin_high(AVR32_PIN_PA08)
#define LED2OFF 	gpio_set_pin_high(AVR32_PIN_PA07)
#define LED1TOGGLE	gpio_toggle_pin(AVR32_PIN_PA08)
#define LED2TOGGLE	gpio_toggle_pin(AVR32_PIN_PA07)

#define MAX_SAMPLE_LIMIT	11900

void spi_init_module(void);
void set_write_latch(void);
void reset_write_latch(void);
uint8_t read_status_register(void);
void write_data(uint32_t address, uint8_t data);
uint8_t read_data(uint32_t address);
void write_mapping(uint16_t adc_value,uint32_t gps_lat, uint32_t gps_lon);
void requestData();
void stopData();
void beatHeart();
__attribute__((__interrupt__)) static void usart_int_handler(void);

spi_options_t my_spi_options={SPI_SLAVECHIP_NUMBER,100000, 8, 5, 5, 1, SPI_MODE_0, 1};

usart_options_t usart1_options={		// HOST link
	.baudrate = USART1_SERIAL_BAUDRATE,
	.charlength = USART1_SERIAL_CHAR_LENGTH,
	.paritytype = USART1_SERIAL_PARITY,
	.stopbits = USART1_SERIAL_STOP_BIT,
	.channelmode = USART_NORMAL_CHMODE,
};

usart_options_t usart2_options={		// MAV link
	.baudrate = USART2_SERIAL_BAUDRATE,
	.charlength = USART2_SERIAL_CHAR_LENGTH,
	.paritytype = USART2_SERIAL_PARITY,
	.stopbits = USART2_SERIAL_STOP_BIT,
	.channelmode = USART_NORMAL_CHMODE,
};

uint16_t coords;
uint32_t eepromAddress;
uint8_t received_byte;
uint8_t txdata;
uint8_t rxdata;
uint8_t STATE;
uint8_t WP;
uint8_t LINK;
uint8_t SAMPLERATE;

ringBufS rx_ringBuff;
mavlink_message_t command;

static const gpio_map_t ADC_GPIO_MAP= {
	{AVR32_ADC_AD_3_PIN,AVR32_ADC_AD_3_FUNCTION}
};
static const gpio_map_t SPI_GPIO_MAP = {
	{AVR32_SPI_SCK_0_0_PIN,AVR32_SPI_SCK_0_0_FUNCTION},
	{AVR32_SPI_MISO_0_0_PIN,AVR32_SPI_MISO_0_0_FUNCTION},
	{AVR32_SPI_MOSI_0_0_PIN,AVR32_SPI_MOSI_0_0_FUNCTION},
	{AVR32_SPI_NPCS_0_0_PIN,AVR32_SPI_NPCS_0_0_FUNCTION}
};
static const gpio_map_t USART0_GPIO_MAP = {
	//{AVR32_USART0_CLK_0_1_PIN,AVR32_USART0_CLK_0_1_FUNCTION},
	{AVR32_USART0_TXD_0_0_PIN,AVR32_USART0_TXD_0_0_FUNCTION},
	{AVR32_USART0_RXD_0_0_PIN,AVR32_USART0_RXD_0_0_FUNCTION},
};
static const gpio_map_t USART1_GPIO_MAP = {		// HOST link
	//{AVR32_USART1_CLK_0_PIN,AVR32_USART1_CLK_0_FUNCTION},
	{AVR32_USART1_TXD_0_0_PIN,AVR32_USART1_TXD_0_0_FUNCTION},
	{AVR32_USART1_RXD_0_0_PIN,AVR32_USART1_RXD_0_0_FUNCTION},
};
static const gpio_map_t USART2_GPIO_MAP = {		// MAV link
	//{AVR32_USART2_CLK_0_PIN,AVR32_USART2_CLK_0_FUNCTION},
	{AVR32_USART2_TXD_0_1_PIN,AVR32_USART2_TXD_0_1_FUNCTION},		// changed from AVR32_USART2_TXD_0_*0*_PIN
	{AVR32_USART2_RXD_0_1_PIN,AVR32_USART2_RXD_0_1_FUNCTION},		// changed from AVR32_USART2_RXD_0_*0*_PIN
};

//===================== SPI FUNCTIONS =======================//
uint8_t get_sample_rate(){
	
};
void spi_init_module(void){
	spi_initMaster(SPI_EXAMPLE,&my_spi_options);
	spi_setupChipReg(SPI_EXAMPLE,&my_spi_options,sysclk_get_pba_hz());
	spi_enable(SPI_EXAMPLE);
}

void erase_eeprom(void){
		set_write_latch();
		spi_selectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
		while(!spi_is_tx_ready(SPI_EXAMPLE));
		spi_put(SPI_EXAMPLE,0xC7);
		while(!spi_is_tx_empty(SPI_EXAMPLE));
		spi_unselectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
		gpio_set_pin_high(AVR32_PIN_PA16);
		reset_write_latch();
	}

void set_write_latch(void){
	while(read_status_register()<2){
	spi_selectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	spi_put(SPI_EXAMPLE,0x06);
	while(!spi_is_tx_empty(SPI_EXAMPLE));
	spi_unselectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	gpio_set_pin_high(AVR32_PIN_PA16);
	}
}
void reset_write_latch(void){
	while(read_status_register()!=0){
	spi_selectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	spi_put(SPI_EXAMPLE,0x04);
	while(!spi_is_tx_empty(SPI_EXAMPLE));
	spi_unselectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	gpio_set_pin_high(AVR32_PIN_PA16);
	}
}
uint8_t read_status_register(void){
	spi_selectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	spi_put(SPI_EXAMPLE,0x05);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	spi_put(SPI_EXAMPLE,0x00);
	while(!spi_is_tx_empty(SPI_EXAMPLE));
	while(!spi_is_rx_ready(SPI_EXAMPLE));
	rxdata = spi_get(SPI_EXAMPLE);
	spi_unselectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	gpio_set_pin_high(AVR32_PIN_PA16);
	return rxdata;
}
void write_data(uint32_t address, uint8_t data){
	spi_init_module();
	set_write_latch();
	spi_selectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	spi_put(SPI_EXAMPLE,0x02);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	txdata=address>>16;
	spi_put(SPI_EXAMPLE,txdata);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	txdata=address>>8;
	spi_put(SPI_EXAMPLE,txdata);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	txdata=address;
	spi_put(SPI_EXAMPLE,txdata);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	txdata=data;
	spi_put(SPI_EXAMPLE,txdata);
	while(!spi_is_tx_empty(SPI_EXAMPLE));
	spi_unselectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	gpio_set_pin_high(AVR32_PIN_PA16);
//	delay_ms(1);
	reset_write_latch();
}
void write_data_line(uint16_t address, char* data, int numBytes){
	
	spi_init_module();
	set_write_latch();
	read_status_register();
	spi_selectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	spi_put(SPI_EXAMPLE,0x02);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	txdata=address>>8;
	spi_put(SPI_EXAMPLE,txdata);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	txdata=address;
	spi_put(SPI_EXAMPLE,txdata);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	for(int i=0;i<numBytes;i++){
	txdata=data[i];
	spi_put(SPI_EXAMPLE,txdata);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	}
	while(!spi_is_tx_empty(SPI_EXAMPLE));
	spi_unselectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	gpio_set_pin_high(AVR32_PIN_PA16);
	reset_write_latch();
	read_status_register();
}

uint8_t read_data(uint32_t address){
	spi_selectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	spi_put(SPI_EXAMPLE,0x03);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	//txdata=address>>8;
	spi_put(SPI_EXAMPLE,(address>>16));
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	spi_put(SPI_EXAMPLE,(address>>8));
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	//txdata=address;
	spi_put(SPI_EXAMPLE,address);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	delay_us(10);
	spi_put(SPI_EXAMPLE,0x00);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	while(!spi_is_tx_empty(SPI_EXAMPLE));
	while(!spi_is_rx_ready(SPI_EXAMPLE));
	rxdata=spi_get(SPI_EXAMPLE);
	spi_unselectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	gpio_set_pin_high(AVR32_PIN_PA16);
	return rxdata;
}
uint8_t read_data_line(uint32_t address,int numBytes){
	spi_selectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	spi_put(SPI_EXAMPLE,0x03);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	//txdata=address>>8;
	spi_put(SPI_EXAMPLE,(address>>8));
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	//txdata=address;
	spi_put(SPI_EXAMPLE,address);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	for(int i=0;i<numBytes;i++){
	spi_put(SPI_EXAMPLE,0x00);
	while(!spi_is_tx_ready(SPI_EXAMPLE));
	}
	spi_unselectChip(SPI_EXAMPLE, SPI_SLAVECHIP_NUMBER);
	gpio_set_pin_high(AVR32_PIN_PA16);
	return rxdata;
}
void write_mapping(uint16_t adc_value, uint32_t gps_lat, uint32_t gps_lon){
	uint8_t mappingData;
	//set_write_latch();
	for(int i=2;i>0;i--){
		mappingData= adc_value>>(8*(i-1));
		write_data(eepromAddress, mappingData);
		eepromAddress++;
	}
	for(int i=4;i>0;i--){
		mappingData= gps_lat>>(8*(i-1));
		write_data(eepromAddress, mappingData);
		eepromAddress++;
	}
	for(int i=4;i>0;i--){
		mappingData= gps_lon>>(8*(i-1));
		write_data(eepromAddress, mappingData);
		eepromAddress++;
	}
	write_data(eepromAddress,'\n');
		eepromAddress++;
		
	write_data(0x000000,((eepromAddress-3)/11)>>8);
	write_data(0x000001,((eepromAddress-3)/11));
}

//===================== MAVLINK FUNCTIONS =======================//
void requestData()
{	
	mavlink_msg_request_data_stream_send(MAVLINK_COMM_0, PIX_SYSID, PIX_CMPID,  MAV_DATA_STREAM_EXTENDED_STATUS, 0x02, STREAM_START);		// THIS FOR GPS CORDS
}
void stopData()
{
	mavlink_msg_request_data_stream_send(MAVLINK_COMM_0, PIX_SYSID, PIX_CMPID,  MAV_DATA_STREAM_ALL, 0x02, STREAM_STOP);
}

int main (void)
{
	sysclk_init();				//Initializes Clock
	board_init();				//Initializes Board
	spi_init_module();			//Initializes SPI0
	usart_serial_init(USART1_HOSTLINK, &usart1_options);		//Initializes USART1 (HOST)
	usart_serial_init(USART2_MAVLINK, &usart2_options);			//Initializes USART2 (MAV)
	
	gpio_enable_module(SPI_GPIO_MAP,sizeof(SPI_GPIO_MAP)/sizeof(SPI_GPIO_MAP[0]));				//enables SPI0 pins
	gpio_enable_module(USART1_GPIO_MAP,sizeof(USART1_GPIO_MAP)/sizeof(USART1_GPIO_MAP[0]));		//enables USART1 Pins
	gpio_enable_module(USART2_GPIO_MAP,sizeof(USART2_GPIO_MAP)/sizeof(USART2_GPIO_MAP[0]));		//enables USART2 Pins
	
	AVR32_ADC.mr |= 0x4 << AVR32_ADC_MR_PRESCAL_OFFSET;									//Sets Prescaler for ADC clk
	gpio_enable_module(ADC_GPIO_MAP,sizeof(ADC_GPIO_MAP)/sizeof(ADC_GPIO_MAP[0]));		//Enables ADC Pin
	adc_configure(&AVR32_ADC);															//Initializes ADC
	adc_enable(&AVR32_ADC,AVR32_ADC_CH3);												//Enables channel 3 of the ADC
	
	gpio_configure_pin(AVR32_PIN_PA07,GPIO_DIR_OUTPUT);
	gpio_configure_pin(AVR32_PIN_PA08,GPIO_DIR_OUTPUT);
	LED1OFF; LED2OFF;
	
	Disable_global_interrupt();						
	 INTC_init_interrupts();
	 INTC_register_interrupt(&usart_int_handler, USART2_MAV_IRQ, AVR32_INTC_INT0);	// Register handler - IRQ is interrupt, INT0 is priority
     (USART2_MAVLINK)->ier =AVR32_USART_IER_RXRDY_MASK;	// Enable USART Rx interrupt.
    Enable_global_interrupt();							// Enable all interrupts.
	
	ringBufS_init(&rx_ringBuff);						// initialize ring buffer  
	
	int pcLinkFlag;
	uint32_t storedCount=0;
	
	int mavChar;
	int beatCount=0;
	uint16_t lastMissionWP=0;
	uint8_t currentFlightMode=0;
	uint8_t curMissionWP=0;
	uint8_t reachedMissionWP=0;
			
	mavlink_system.sysid = 255;
	mavlink_system.compid = 1;
	mavlink_message_t msg;
	mavlink_status_t status;

	stopData();							// reset mavlink stream to known state
	requestData();
	
	eepromAddress = 0x000004;			//Initializes Memory address to 0x0001
	//erase_eeprom();
	
	
	STATE=IDLE;							//Initializes to IDLE State
	
	//////////////////////////////////////////////////////////////////
	/*while(1) {
		adc_start(&AVR32_ADC);
		uint32_t adc_value_temp = adc_get_value(&AVR32_ADC,AVR32_ADC_CH3);
		
		usart_serial_putchar(USART1_HOSTLINK, adc_value_temp);
		usart_serial_putchar(USART1_HOSTLINK, (adc_value_temp>>8));
		usart_serial_putchar(USART1_HOSTLINK, '\r');
		delay_ms(5);
	}*/
	/////////////////////////////////////////////////////////////////
	
	//write_data(0x000000, 25);
			
	while(1){
		//State machine starts
		// LED INDICATORS:
		// IDLE:		ONE ON / ONE OFF
		// SAMPLING:	BOTH TOGGLE
		// LINK:		BOTH ON
		
		switch(STATE){
			//IDLE state checks for either PC Link Flag or confirmation that First WP is reached.
			case IDLE: {
				while(STATE==IDLE){
					LED1ON;
					LED2OFF;
					if(usart_read_char(USART1_HOSTLINK, &pcLinkFlag)==USART_SUCCESS) {	//If host PC link char state goes to LINKED
						if(pcLinkFlag=='g')	{
							STATE=LINKED;
							break;		}		 // break out of while
					}
					if (!ringBufS_empty(&rx_ringBuff))	{								// if char in buffer, process
						mavChar=ringBufS_get(&rx_ringBuff);								// pop character
						//usart_serial_putchar(USART1_HOSTLINK, mavChar);
						if(mavlink_parse_char(MAVLINK_COMM_0,mavChar,&msg,&status))	{
							switch(msg.msgid){
								case MAVLINK_MSG_ID_HEARTBEAT:
									currentFlightMode = (mavlink_msg_heartbeat_get_custom_mode(&msg)>>24);		// update flight mode
									if (lastMissionWP==0 && beatCount>2) {										// every two heartbeats, request missionCount if zero
										mavlink_msg_mission_request_list_send(MAVLINK_COMM_0,PIX_SYSID,PIX_CMPID);		
										beatCount=0;
									}
									beatCount++;
									break;
								case MAVLINK_MSG_ID_MISSION_COUNT:												// get total number of waypoints/mission items
									lastMissionWP = (mavlink_msg_mission_count_get_count(&msg)>>8)-2;			// calc lastWP; number of items including 0:home cords. M.Planner list is 1 to (numItems-1)
									break;
								case MAVLINK_MSG_ID_MISSION_ITEM_REACHED:
									reachedMissionWP = mavlink_msg_mission_item_reached_get_seq(&msg)>>8;
									if ((reachedMissionWP >= 2) && (storedCount < MAX_SAMPLE_LIMIT) && (storedCount==0))
										STATE=SAMPLING;
									break;
								default:
									break;
							} // end switch
						} // end of parse GOOD
					} // end of buffer !empty
				} // end of IDLE while
			} // end of case IDLE
			break;
			case SAMPLING:{
				uint32_t lat;
				uint32_t lon;
				uint32_t alt;
				uint16_t adc_value=0;
				uint16_t adc_value_highest=0;
				adc_start(&AVR32_ADC);

				while(STATE==SAMPLING)	{			//Starts monitoring MAVLINK Messages from the flight controller
					LED1TOGGLE;
					LED2TOGGLE;
					
					adc_start(&AVR32_ADC);
					adc_value = adc_get_value(&AVR32_ADC,AVR32_ADC_CH3);	//Signal Intensity
					if (adc_value > adc_value_highest){
						adc_value_highest = adc_value;
					}
					if (!ringBufS_empty(&rx_ringBuff))	{								// if char in buffer, process
						mavChar=ringBufS_get(&rx_ringBuff);								// pop character
						if(mavlink_parse_char(MAVLINK_COMM_0,mavChar,&msg,&status)) {
							switch(msg.msgid)	{
								case MAVLINK_MSG_ID_HEARTBEAT:
									currentFlightMode = (mavlink_msg_heartbeat_get_custom_mode(&msg)>>24);				// update flight mode
									if ((currentFlightMode==FLTMODE_RTL) || (currentFlightMode==FLTMODE_LAND)){		// in case of emergency return or land, IDLE
										STATE=IDLE;
									}
									if (lastMissionWP==0 && beatCount>2){							// if mission count not received yet (should be in idle), request
										mavlink_msg_mission_request_list_send(MAVLINK_COMM_0,PIX_SYSID,PIX_CMPID);
										beatCount=0;
									}
									beatCount++;
									break;
								/*case MAVLINK_MSG_ID_MISSION_CURRENT:								// check for mission completion - go to IDLE if done
									curMissionWP=mavlink_msg_mission_current_get_seq(&msg);
									if((lastMissionWP!=0) && (curMissionWP > lastMissionWP)) {		// only exit if last waypoint number was received
										STATE=IDLE;
									}
									break;*/
								case MAVLINK_MSG_ID_MISSION_ITEM_REACHED:
									reachedMissionWP = mavlink_msg_mission_item_reached_get_seq(&msg)>>8;
									if ((reachedMissionWP >= (lastMissionWP)) && (lastMissionWP != 0))		// added -1 to test at serenas
										STATE=IDLE;
									break;									
								case MAVLINK_MSG_ID_MISSION_COUNT:									// should be received by now, but just in case!
									lastMissionWP = mavlink_msg_mission_count_get_count(&msg)-2;	// number of items including 0:home cords. M.Planner list is 1 to (numItems-1)
								break;
								case MAVLINK_MSG_ID_GPS_RAW_INT:									//If GPS Message is received: lat, lon, and signal intensities are sampled and mapped
									lat=mavlink_msg_gps_raw_int_get_lat(&msg);		// Lat
									lon=mavlink_msg_gps_raw_int_get_lon(&msg);		// Lon
									alt=mavlink_msg_gps_raw_int_get_alt(&msg);		// Lon
									//adc_start(&AVR32_ADC);	
									//adc_value = adc_get_value(&AVR32_ADC,AVR32_ADC_CH3);	//Signal Intensity
									write_mapping(adc_value_highest, lat,lon);			//Write data to storage
									adc_value_highest = 0;
									//delay_ms(500);
									if (storedCount++ >= MAX_SAMPLE_LIMIT)						// exit to IDLE if we've stored to limit
									{	STATE=IDLE;   }
									break;
								default:
									break;
							} //end of switch(msd.id)
						}	// end of if (parseOK)
					}	// end of buffer !empty
				} //end of while (sampling)
			break;
			} //end of case
			
			case LINKED: {		//Linked
				uint8_t c;
				uint32_t addressCounter=0x00000004;
				uint16_t numBytes=0x00;
				numBytes = numBytes | ((uint16_t) read_data(0x000000)<<8);
				numBytes = numBytes | read_data(0x000001);
				LED1ON;
				LED2ON;

				//Sends signal to host PC that data is incoming
				usart_serial_putchar(USART1_HOSTLINK,'h');
				usart_serial_putchar(USART1_HOSTLINK,numBytes>>8);
				usart_serial_putchar(USART1_HOSTLINK,numBytes);
					
				while(addressCounter< ((numBytes*11)+4) )	{
					if(usart_serial_putchar(USART1_HOSTLINK,read_data(addressCounter))){
						usart_serial_getchar(USART1_HOSTLINK,&c);
						if(c=='k'){
							addressCounter++;
						}
					}
				}
				usart_serial_putchar(USART1_HOSTLINK,'\0');
				STATE=IDLE;
				LED1OFF;
				LED2OFF;
				break;
			} // end Case: LINKED
			case TEST:
			{
				uint16_t adc_value=0;
				while(1){
					adc_start(&AVR32_ADC);
					adc_value = adc_get_value(&AVR32_ADC,AVR32_ADC_CH3);
					usart_serial_putchar(USART1_HOSTLINK, adc_value>>8);
					usart_serial_putchar(USART1_HOSTLINK, adc_value);
					usart_serial_putchar(USART1_HOSTLINK, 0xFE);
					delay_ms(100);
				}
			}
			break;
		} // end switch(STATE)
	} // end WHILE(1)
} // end MAIN()

 __attribute__((__interrupt__)) static void usart_int_handler(void)
{
	int c,status;
	if ((USART2_MAVLINK)->csr & AVR32_USART_CSR_RXRDY_MASK) {			// if *RX* interrupt
		if (usart_read_char(USART2_MAVLINK, &c) == USART_SUCCESS)
		 {	ringBufS_put(&rx_ringBuff, (unsigned char)c);	 }
		else
		 {	usart_reset_status(USART2_MAVLINK);	}
		//usart_write_char(USART1_HOSTLINK,rx_ringBuff.count);	
	}
 }